@@ -95,14 +95,14 @@ int read_access(const char* filename) {
 char* read_line(FILE* file) {
 	char buffer[LINESIZE];
 	char* line, *line2;
-	int linesize;
+	size_t linesize;
 
 	line = (char*) malloc(1);
 	enough_mem(line);
 	line[0] = '\0';
 	while (fgets(buffer, LINESIZE, file) == buffer
 		&& buffer[0] != '\0') {
-		
+
 		line2 = line;
 		linesize = LINESIZE + strlen(line2) + 1;
 		line = (char*) malloc(linesize);
@@ -259,8 +259,9 @@ struct nameinfo* get_names(char* line) {
 	char* p = line;
 	char* buf, *tmp;
 	char n[NAMESIZE];
-	int i, bufsize;
-	
+	int i;
+	size_t bufsize;
+
 	for (;;) {
 		while(*p && !isalpha((int)*p)) {
 			/* see if it is a domain like .ibm.com */
@@ -519,7 +520,7 @@ int is_ip_target(const char* source_ip,
 				return 1;
 			}
 			if (destuser) {
-				int dusize = strlen(destuser) + 2;
+				size_t dusize = strlen(destuser) + 2;
 				du = (char*) malloc(dusize);
 				snprintf(du, dusize, ">%s", destuser);
 				if (is_user_allowed(du, allowed)) {
@@ -561,7 +562,7 @@ int is_ip_target(const char* source_ip,
 					return 1;
 				}
 				if (destuser) {
-					int dusize = strlen(destuser) + 2;
+					size_t dusize = strlen(destuser) + 2;
 					du = (char*) malloc(dusize);
 					snprintf(du, dusize, ">%s", destuser);
 					if (is_user_allowed(du, allowed)) {
