@@ -47,8 +47,6 @@ int handlecmds(const char* hostnames, struct clientinfo *clntinfo) {
 	char *buffer = 0;
 	int ss, cs, ret;
 	int i;
-	unsigned long int iaddr;
-	struct hostent *host;
 
 	conn_info.lsforward = &lsforward;
 	conn_info.li = &li;
@@ -64,19 +62,6 @@ int handlecmds(const char* hostnames, struct clientinfo *clntinfo) {
 	}
 	cs = clntinfo->clientsocket;
 
-	/* use li.host as a temporary pointer */
-	li.host = getpeer_ip(clntinfo->clientsocket);
-	iaddr = inet_addr(li.host);
-	free(li.host);
-	host = gethostbyaddr((char*) &iaddr, sizeof(iaddr), AF_INET);
-	if (host) {
-		li.host = strdup(host->h_name);
-	} else {
-		struct in_addr in;
-		in.s_addr = clntinfo->clientaddr;
-		li.host = strdup(inet_ntoa(in));
-	}
-	enough_mem(li.host);
 
 	li.user = 0;
 	li.name = (char*) 0;
@@ -177,7 +162,8 @@ contin:
 			"500 Command not understood or not implemented.\r\n");
 		}
 	}
-	/* li.host and li.user  are freed at the termination of the programm */
+	/* li.client, li.server and li.user  are freed at the termination of
+	 * the programm */
 }
 
 
@@ -214,7 +200,7 @@ int getuserdest(char* buffer, struct clientinfo *clntinfo) {
 
 	opt = get_option("passallauth");
 	if (opt) {
-		int bufsize;
+		size_t bufsize;
 		/* passallauth specified */
 		/* transform buffer into "buffer"@targethost,21 */
 
@@ -280,6 +266,7 @@ int getuserdest(char* buffer, struct clientinfo *clntinfo) {
 	if (buffer[i-1] == '\0') {
 		const char* tmp;
 		const char* addtarget = 0;
+		int transproxy = 1;
 		char ipbuf[ IPBUFSIZE ];
 		/* first, lets terminate user. Since it is the last
 		 * value in the line, it is followed by an \r\n */
@@ -288,6 +275,24 @@ int getuserdest(char* buffer, struct clientinfo *clntinfo) {
 		}
 		tmp = get_option("defaultforward");
 		if (!tmp) {
+			if (get_option("transparent-proxy")) {
+				if (strcmp(get_option("transparent-proxy"),
+							"on") == 0
+					||
+				    strcmp(get_option("transparent-proxy"),
+							"1") == 0
+					||
+				    strcmp(get_option("transparent-proxy"),
+							"yes") == 0) {
+					transproxy = 1;
+				} else {
+					transproxy = 0;
+				}
+			} else {
+				transproxy = 0;
+			}
+		}
+		if (transproxy) {
 			struct sockaddr_in sin;
 			const char* fwhost = 0;
 #ifdef HAVE_SOCKLEN_T
@@ -327,7 +332,7 @@ int getuserdest(char* buffer, struct clientinfo *clntinfo) {
 				addtarget = fwhost;
 				log(8, "Using transparent proxy. Connecting to %s. User: %s", addtarget, user);
 			} else {
-				int destsize;
+				size_t destsize;
 				log(8, "Connecting to %s as transparent proxy",
 					tmp);
 				/* resize dest */
@@ -340,12 +345,18 @@ int getuserdest(char* buffer, struct clientinfo *clntinfo) {
 				addtarget = 0;
 			}
 		} else {
+			if (!tmp) {
+				/* No defaultroute specified and no
+				 * transparent proxy support. Kick it */
+				say(cs, "500 You are not allowed to connect\r\n");
+				return -1;
+			}
 			addtarget = tmp;
 		}
 		log(9, "Addtarget: %s", addtarget);
 		if (addtarget) {
 			/* copy the option of defaultforward into buffer */
-			int buffnewsize = strlen(buffer)
+			size_t buffnewsize = strlen(buffer)
 						+ strlen(addtarget)
 						+ 1
 						+ 1;
@@ -488,7 +499,7 @@ int passcmd(char* buffer, struct clientinfo *clntinfo) {
 	char* sendbuf =0;
 	char *last = 0;
 	int ret;
-	int sendbufsize;
+	size_t sendbufsize;
 	int transfer_start;
 
 	sendbufsize = strlen(buffer) + 3;
