@@ -31,7 +31,8 @@ char* valid_keys[] = { "accessfile", "serverport", "defaultmode",
 			"transparent-forward", "pidfile", "welcomeline",
 			"listen", "debuglevel", "allowreservedports",
 			"allowforeignaddress", "passiveportrange",
-			"activeportrange", "logstyle", (char*) 0 };
+			"activeportrange", "logstyle", "transparent-proxy",
+			(char*) 0 };
 
 struct optionstruct {
 	char* key;
@@ -681,7 +682,7 @@ float get_rate(const char *fwuser, const char* user,
 	}
 
 	if (user) {
-		int arrusersize = strlen(user) + 2;
+		size_t arrusersize = strlen(user) + 2;
 		/* user may be (char*) 0 if no forward is used */
 		arruser = malloc(arrusersize);
 		enough_mem(arruser);
@@ -1111,7 +1112,7 @@ int add_rangespec(const char* entry) {
 	unsigned int *portsamount;
 	struct portrangestruct** prsbase, **prscur, *prs;
 	char* startportstr, *endportstr;
-	unsigned int startport, endport;
+	long startport, endport;
 	char* typestr = quotstrtok(entry, WHITESPACES"\n", &offset);
 	struct lchar* llistbase = split_line(entry, typestr);
 	struct lchar* llistcur = llistbase;
@@ -1619,7 +1620,7 @@ const char* getaccessfile() {
 	const char* opt = get_option("accessfile");
 	if (!opt) {
 		char *b;
-		int bsize;
+		size_t bsize;
 		fprintf(stderr, "No accessfile option specified. Assuming "
 				"%s as default\n", DEFAULTACCESSFILE);
 		log(4, "No accessfile option specified. Assuming "
