@@ -46,19 +46,21 @@ char* ftp_readline(int fd) {
 
 static char *readline_check(int fd, int check_ftp_format) {
 	const int MAXSIZE = 3;
-	int n, ret, length = 0;
+	const unsigned int LIMIT = UINT_MAX - MAXSIZE;
+	int n, ret;
+	unsigned int length = 0;
 	int linecnt;
 	int data_read = 0;
 	char *temp, *buffer;
 	fd_set readset;
 	struct timeval rtimeout = { 300, 0 };
 	sigset_t sigset, oldset;
-	
+
 	FD_ZERO(&readset);
 	FD_SET(fd, &readset);
 
 	timeout = 0;
-	
+
 	sigemptyset(&sigset);
 	sigemptyset(&oldset);
 	sigaddset(&sigset, SIGCHLD);
@@ -85,7 +87,7 @@ static char *readline_check(int fd, int check_ftp_format) {
 		log(3, "sigprocmask() error releasing the blocked"
 			" signals: %s", strerror(errno));
 	}
-	
+
 	if (fd < 0) {
 		log(1, "readline_check: Not connected");
 		return 0;
@@ -107,13 +109,19 @@ static char *readline_check(int fd, int check_ftp_format) {
 		 * or
 		 * xxx-<text> (with a dash)
 		 */
+		if (length >= LIMIT) {
+			/* input too long */
+			log(1, "Input too long");
+			free(buffer);
+			return (char*) 0;
+		}
 		length++;
 		if (check_ftp_format && linecnt == 0
 			&& ((length <= 3 && !isdigit((int) *temp))
 				|| (length == 4 && *temp != ' '
 						&& *temp != '-'))) {
 
-				buffer[length] = 0;
+				buffer[length] = (char) 0;
 				log(4, "malformed FTP response: %s",
 					buffer);
 				log(9, "in line: %d", linecnt);
@@ -278,7 +286,7 @@ struct message readall(int sourcefd) {
 
 char* passall(int sourcefd, int targetfd) {
 	char* line =0, *sendbuf =0;
-	int sendbufsize;
+	size_t sendbufsize;
 
 	while ((line = readline(sourcefd))) {
 		if (!*line) {
