@@ -48,6 +48,7 @@ int chrooted = 0;
 extern int should_read_config;
 char *conffilename;
 char *binaryname;
+unsigned int number_of_children = 0;
 
 
 static struct option const long_option_arr[] =
@@ -113,7 +114,7 @@ int main(int argc, char** argv) {
 	}
 
 	/* Read the configuration */
-	config.debuglevel = 9;
+	config.debuglevel = INITIAL_DEBUGLEVEL;
 	config.logf_name = (char*) 0;
 	config.logf = (FILE*) 0;
 
@@ -285,11 +286,12 @@ int init_logfiles(const char* dir_prefix) {
 	const char* option;
 	const char* specs;
 	const char* style;
+	const char* log_peer;
 	char* cpy;
 	const int MIDLENGTH = 64;
 	struct cmdlogent_t* cmdlogfilescur = &config.cmdlogfiles;
 	struct cmdlogent_t* cmdlogdirscur = &config.cmdlogdirs;
-	int logf_name_size;
+	size_t logf_name_size;
 
 	if (!dir_prefix) {
 		dir_prefix = "";
@@ -368,7 +370,7 @@ int init_logfiles(const char* dir_prefix) {
 			break;
 		}
 		else {
-			int cpysize;
+			size_t cpysize;
 			snprintf(buf, sizeof(buf), "cmdlogfile%d-specs", i);
 			specs = get_option(buf);
 			if (strlen(dir_prefix)) {
@@ -412,6 +414,22 @@ int init_logfiles(const char* dir_prefix) {
 						style, buf);
 				cmdlogfilescur->style = LS_COMMONLOG;
 			}
+			snprintf(buf, sizeof(buf),
+					"cmdlogfile%d-log", i);
+			log_peer = get_option(buf);
+			if (!log_peer) {
+				/* use client as default */
+				cmdlogfilescur->log_peer = LS_LOG_PEER_CLIENT;
+			} else if (strcasecmp(log_peer, "client") == 0) {
+				cmdlogfilescur->log_peer = LS_LOG_PEER_CLIENT;
+			} else if (strcasecmp(log_peer, "server") == 0) {
+				cmdlogfilescur->log_peer = LS_LOG_PEER_SERVER;
+			} else {
+				/* default */
+				log(4, "Invalid log option %s for %s",
+						log_peer, buf);
+				cmdlogfilescur->log_peer = LS_LOG_PEER_CLIENT;
+			}
 
 			cmdlogfilescur->next = 0;
 			if ((cmdlogfilescur->logf = 
@@ -435,7 +453,7 @@ int init_logfiles(const char* dir_prefix) {
 			break;
 		} else {
 			const char* prefix =0, *suffix =0;
-			int size;
+			size_t size;
 			snprintf(buf, sizeof(buf),
 					"connectionlogdir%d-specs", i);
 			specs = get_option(buf);
@@ -501,6 +519,23 @@ int init_logfiles(const char* dir_prefix) {
 				cmdlogdirscur->style = LS_COMMONLOG;
 			}
 
+			snprintf(buf, sizeof(buf),
+					"connectionlogdir%d-log", i);
+			log_peer = get_option(buf);
+			if (!log_peer) {
+				/* use client as default */
+				cmdlogdirscur->log_peer = LS_LOG_PEER_CLIENT;
+			} else if (strcasecmp(log_peer, "client") == 0) {
+				cmdlogdirscur->log_peer = LS_LOG_PEER_CLIENT;
+			} else if (strcasecmp(log_peer, "server") == 0) {
+				cmdlogdirscur->log_peer = LS_LOG_PEER_SERVER;
+			} else {
+				/* default */
+				log(4, "Invalid log option %s for %s",
+						log_peer, buf);
+				cmdlogdirscur->log_peer = LS_LOG_PEER_CLIENT;
+			}
+
 			cmdlogdirscur->next = 0;
 		}
 	}
@@ -620,9 +655,8 @@ void closedescriptors(void) {
 	 * itself is on the stack */
 	/* li.cmd must not be freed, it's   li.cmd = buffer;  */
 	/* the same for li.name */
-	if (li.host) {
-		free(li.host);
-	}
+	if (li.client) { free(li.client); }
+	if (li.server) { free(li.server); }
 	if (li.user) {
 		free(li.user);
 	}
