@@ -100,7 +100,7 @@ void log(int level, char *fmt, ...)
 		if (level < 4) {
 			logtype = LOG_ERR;
 		}
-		vsnprintf(str, 800 - 1, fmt, args);
+		vsnprintf(str, LOGSIZE - 1, fmt, args);
 		syslog(logtype, str);
 	}
 
@@ -110,7 +110,7 @@ void log(int level, char *fmt, ...)
 void log_cmd_ent(struct cmdlogent_t* lent, struct loginfo* li) {
 
 	char* commandpattern;
-	int commandpatternsize;
+	size_t commandpatternsize;
 	static char time_string[LENGTH];
 	time_t nowtime;
 	char* ws;
@@ -132,6 +132,14 @@ void log_cmd_ent(struct cmdlogent_t* lent, struct loginfo* li) {
 
 	if (strstr(lent->specs, commandpattern)) {
 		/* found, log it */
+		const char* host;
+
+		if (lent->log_peer == LS_LOG_PEER_SERVER) {
+			host = li->server;
+		} else {
+			host = li->client;
+		}
+
 		nowtime = time(NULL);
 
 		if (lent->style == LS_COMMONLOG) {
@@ -140,7 +148,7 @@ void log_cmd_ent(struct cmdlogent_t* lent, struct loginfo* li) {
 				"[%d/%b/%Y:%H:%M:%S %Z]",
 				localtime(&nowtime));
 			fprintf(lent->logf, "%s UNKNOWN %s %s ",
-				li->host, li->user, time_string);
+				host, li->user, time_string);
 			if (checkbegin(li->cmd, "PASS")) {
 				fprintf(lent->logf, "\"PASS *\"");
 			} else {
@@ -156,7 +164,7 @@ void log_cmd_ent(struct cmdlogent_t* lent, struct loginfo* li) {
 				"%a %b %d %H:%M:%S %Y",
 				localtime(&nowtime));
 			fprintf(lent->logf, "%s %d %s %ld %s %c _ %c %c %s %s 0 * %c\n",
-					time_string, li->transfer_duration, li->host, li->transferred, li->name, li->type, li->direction, strcmp(li->user, "anonymous") == 0 || strcmp(li->user, "ftp") ? 'a' : 'r', li->user, li->service, li->complete ? 'c' : 'i');
+					time_string, li->transfer_duration, host, li->transferred, li->name, li->type, li->direction, strcmp(li->user, "anonymous") == 0 || strcmp(li->user, "ftp") ? 'a' : 'r', li->user, li->service, li->complete ? 'c' : 'i');
 			li->name = (char*) 0;
 		}
 		fflush(lent->logf);
