@@ -1,3 +1,21 @@
+/* 
+ * Copyright (C) 1999-2001 Joachim Wieland <joe@mcknight.de>
+ * 
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2 of the License, or (at your
+ * option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
+ */
+
 #include <sys/time.h>
 #include "jftpgw.h"
 
@@ -49,7 +67,7 @@ int login(struct clientinfo *clntinfo) {
 	if (!allowed) {
 		say(cs, "530 Login incorrect\r\n");
 		return -1;
-	}	
+	}
 
 	allowed = 0;
 
@@ -123,15 +141,14 @@ int login(struct clientinfo *clntinfo) {
 	if (ret) {
 		/* free the login struct lsforward */
 		free_loginst();
-		
+
 		free(peer_ip_cln);
 		free(peer_ip_svr);
 		return ret;
 	}
-	
+
 	log(6, "Logged in to %s as %s!", clntinfo->destination, clntinfo->user);
-	
-		
+
 	/* get the throughput rate */
 
 	clntinfo->throughput = get_rate(lsforward.fwlogin, lsforward.login,
@@ -155,23 +172,26 @@ int std_login(struct clientinfo* clntinfo) {
 	int cs = clntinfo->clientsocket;
 	int ss = -1;
 	int ret;
+	size_t sendbufsize, bufsize;
 	char* peer_ip =0;
 	char* buffer =0, *sendbuf =0;
 	struct message welcomemsg = { 0, 0 };
 	struct message authresp = { 0, 0 };
 	char codebuf[5];
-
+	unsigned long int iaddr;
+	struct hostent *host;
 
 	ss = openportname(clntinfo->destination, port, 0, clntinfo);
 	if (ss < 0) {
 		int err = errno;
-		sendbuf = (char*) malloc(
-				strlen("Error connecting to %s: %s\r\n")
+		size_t sendbufsize = strlen("Error connecting to %s: %s\r\n")
 				+ strlen(clntinfo->destination)
 				+ strlen(strerror(err))
-				+ 1);
+				+ 1;
+		sendbuf = (char*) malloc(sendbufsize);
 		enough_mem(sendbuf);
-		sprintf(sendbuf, "Error connecting to %s: %s\r\n",
+		snprintf(sendbuf, sendbufsize,
+				"Error connecting to %s: %s\r\n",
 				clntinfo->destination, strerror(err));
 		say(cs, sendbuf);
 		free(sendbuf);
@@ -179,7 +199,7 @@ int std_login(struct clientinfo* clntinfo) {
 	}
 
 	peer_ip = getpeer_ip(ss);
-	clntinfo->serveraddr = get_showaddr(ss);
+	clntinfo->serveraddr = get_showaddr(ss, SHOW_NOT_ORIGINAL_IP);
 	free(peer_ip);
 	clntinfo->serversocket = ss;
 
@@ -192,17 +212,18 @@ int std_login(struct clientinfo* clntinfo) {
 		} else {
 			err_readline(cs);
 		}
-		return -1;
+		return -2;
 	}
 
 	log(9, "Connected to %s, got \"%s\" as welcome line", clntinfo->destination, welcomemsg.fullmsg);
-	
+
 	if (!checkbegin(welcomemsg.lastmsg, "220 ")) {
 		log(1, "Not a valid FTP server response (%s)", welcomemsg.fullmsg);
 		say(cs, welcomemsg.fullmsg);
+		free(welcomemsg.fullmsg);
 		exit(1);
 	}
-	
+
 	say(ss, "SYST\r\n");
 	buffer = ftp_readline(ss);
 	if (!buffer && timeout) {
@@ -212,6 +233,7 @@ int std_login(struct clientinfo* clntinfo) {
 	}
 	if (!buffer) {
 		err_readline(cs);
+		/* -2 ?? */
 		return -1;
 	}
 	if (!strlen(buffer)) {
@@ -220,13 +242,13 @@ int std_login(struct clientinfo* clntinfo) {
 		return -1;
 	}
 	free(buffer);
-	buffer = 0;
-	
-	sendbuf = (char*) malloc(strlen("USER \r\n")
-			+ strlen(clntinfo->user) + 1);
+	buffer = (char*) 0;
+
+	sendbufsize = strlen("USER \r\n") + strlen(clntinfo->user) + 1;
+	sendbuf = (char*) malloc(sendbufsize);
 	enough_mem(sendbuf);
 
-	sprintf(sendbuf, "USER %s\r\n", clntinfo->user);
+	snprintf(sendbuf, sendbufsize, "USER %s\r\n", clntinfo->user);
 	ret = say(ss, sendbuf);
 	free(sendbuf);
 	sendbuf = 0;
@@ -241,36 +263,41 @@ int std_login(struct clientinfo* clntinfo) {
 	if (!buffer && timeout) {
 			log(2, "Timeout in %s line %d\n", __FILE__ ,__LINE__);
 		err_time_readline(cs);
+		free(welcomemsg.fullmsg);
 		return -2;
 	}
 	if (!buffer) {
 		err_readline(cs);
+		free(welcomemsg.fullmsg);
 		return -1;
 	}
 	if (!checkdigits(buffer, 331) && !checkdigits(buffer, 230)) {
 		say(cs, buffer);
 		log(7, "Got \"%s\" after sending the username.", buffer);
-		free(buffer);
+		free(authresp.fullmsg);
+		free(welcomemsg.fullmsg);
 		return -2;
 	}
-	
+
 	if (!checkdigits(buffer, 230)) {
-		sendbuf = (char*) malloc(strlen("PASS \r\n") 
-					  + strlen(clntinfo->pass) + 1);
+		sendbufsize = strlen("PASS \r\n") + strlen(clntinfo->pass) + 1;
+		sendbuf = (char*) malloc(sendbufsize);
 		enough_mem(sendbuf);
-		sprintf(sendbuf, "PASS %s\r\n", clntinfo->pass);
+		snprintf(sendbuf, sendbufsize, "PASS %s\r\n", clntinfo->pass);
 		ret = say(ss, sendbuf);
 		free(sendbuf);
-		sendbuf = 0;
+		sendbuf = (char*) 0;
 		if (ret < 0) {
 			log(2, "Error writing the password to the server: %s",
 				strerror(errno));
+			free(welcomemsg.fullmsg);
+			free(authresp.fullmsg);
 			return -1;
 		}
+		free(authresp.fullmsg);
 
 		authresp = readall(ss);
 		if (!authresp.fullmsg) {
-			free(welcomemsg.fullmsg);
 			if (timeout) {
 				log(2, "Timeout in %s line %d\n", __FILE__ ,
 						__LINE__); 
@@ -279,35 +306,37 @@ int std_login(struct clientinfo* clntinfo) {
 			else {
 				err_readline(cs);
 			}
+			free(welcomemsg.fullmsg);
 			return -1;
 		}
 	}
-	free(buffer);
 
-	buffer = (char*) malloc(strlen(welcomemsg.fullmsg)
-			+ strlen(authresp.fullmsg) + 1);
+	bufsize = strlen(welcomemsg.fullmsg) + strlen(authresp.fullmsg) + 1;
+	buffer = (char*) malloc(bufsize);
 	enough_mem(buffer);
-	
+
 	/* change the status code of the welcomemsg */
-	
+
 	/* save the digit code of the server response from the
 	 * authentication in codebuf */
 	strncpy(codebuf, authresp.fullmsg, 4);
 	codebuf[3] = '-';
 	codebuf[4] = '\0';
-	
+
 	/* Change the digit code of the welcome message to the one of the
 	 * server response from the authentication */
 	changecode(welcomemsg.fullmsg, codebuf);
-	strcpy(buffer, welcomemsg.fullmsg);
-	strcat(buffer, authresp.fullmsg);
-	
+	snprintf(buffer, bufsize, "%s%s",
+			welcomemsg.fullmsg, authresp.fullmsg);
+
 	li.respcode = getcode(authresp.lastmsg);
-	
+
 	if (!checkdigits(authresp.lastmsg, 230)) {
 		say(cs, authresp.lastmsg);
 		log(8, "Got \"%s\" after sending the password", authresp.fullmsg);
-		return -2;	
+		free(welcomemsg.fullmsg);
+		free(authresp.fullmsg);
+		return -2;
 	}
 
 	/* successful login */
@@ -325,6 +354,41 @@ int std_login(struct clientinfo* clntinfo) {
 	buffer =0;
 
 
+	/* use li.client as a temporary pointer */
+	li.client = getpeer_ip(clntinfo->clientsocket);
+	iaddr = inet_addr(li.client);
+	free(li.client);
+	host = gethostbyaddr((char*) &iaddr, sizeof(iaddr), AF_INET);
+	if (host) {
+		li.client = strdup(host->h_name);
+	} else {
+		struct in_addr in;
+		char* peer = getpeer_ip(clntinfo->clientsocket);
+
+		in.s_addr = inet_addr(peer);
+		li.client = strdup(inet_ntoa(in));
+		free(peer);
+	}
+	enough_mem(li.client);
+
+	/* use li.server as a temporary pointer */
+	li.server = getpeer_ip(clntinfo->serversocket);
+	iaddr = inet_addr(li.server);
+	free(li.server);
+	host = gethostbyaddr((char*) &iaddr, sizeof(iaddr), AF_INET);
+	if (host) {
+		li.server = strdup(host->h_name);
+	} else {
+		struct in_addr in;
+		char* peer = getpeer_ip(clntinfo->clientsocket);
+		in.s_addr = inet_addr(peer);
+		li.server = strdup(inet_ntoa(in));
+		free(peer);
+	}
+	enough_mem(li.server);
+
+	log(9, "client: %s, server: %s", li.client, li.server);
+
 	return 0;
 }
 
@@ -341,7 +405,7 @@ int sftp_login(struct clientinfo* clntinfo) {
 	int pw_match = 0;
 	int nargs;
 	struct in_addr in;
-	
+
 	if (!gethostaddr(clntinfo->destination,
 				&in)) {
 		say(clntinfo->clientsocket, "532 Could not look up the "
