@@ -17,6 +17,7 @@
  */
 
 #include "jftpgw.h"
+#include <ctype.h>
 
 extern int timeout;
 extern struct portrangestruct *pasvrangebase;
@@ -62,20 +63,34 @@ int pasvserver(struct clientinfo* clntinfo) {
 	/* 
 	 * An example of the answer would be:
 	 * "227 Entering Passive Mode (127,0,0,1,7,123)"
+	 *
+	 * Roxen Challenger at ftp.mutt.org sends this reply
+	 * "227 Entering Passive Mode. 217,69,76,44,156,68"
 	 */
 	/* Look for the beginning of the IP */
 	brk = (char*) strchr(buffer, '(');
 
-	if (!checkdigits(buffer, 227) || !brk) {
+	if (!checkdigits(buffer, 227)) {
 		/* invalid return string */
 		say(cs, buffer);
 		log(2, "Strange answer after sending PASV: %s", buffer);
 		free(buffer);
 		return -1;
 	}
-	
-	/* skip over the bracket sign */
-	brk++;
+	if (!brk) {
+		/* don`t give up, maybe it just didn`t send a bracket */
+		/* skip over the numerical code */
+		brk = strchr(buffer, ' ');
+		brk++;
+		/* increase until we found a numeric */
+		while (*brk && !isdigit((int)*brk)) {
+			brk++;
+		}
+		/* now brk should point to the first numeric */
+	} else {
+		/* skip over the bracket sign */
+		brk++;
+	}
 
 	/* call parsesock to parse the answer string and get the values into
 	 * PASVSERVERSOCK */
