@@ -1,3 +1,21 @@
+/* 
+ * Copyright (C) 1999-2001 Joachim Wieland <joe@mcknight.de>
+ * 
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2 of the License, or (at your
+ * option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
+ */
+
 #include "jftpgw.h"
 #ifdef SFTP_SUPPORT
 #include "cmds.h"
@@ -116,6 +134,7 @@ int secsftp_pwd(char* args, struct conn_info_st* conn_info) {
 		char* out =0;
 		char* msg =0; 
 		char* sendbuf =0;
+		size_t sendbufsize;
 		int ret;
 		ret = do_pwd(arrlist.arr, arrlist.amount);
 		switch (ret) {
@@ -128,8 +147,9 @@ int secsftp_pwd(char* args, struct conn_info_st* conn_info) {
 				conn_info->li->respcode = 500;
 		}
 		out = double_quote(sftp_out_buffer);
-		sendbuf = (char*) malloc(strlen(msg) + strlen(out) + 1);
-		sprintf(sendbuf, msg, out);
+		sendbufsize = strlen(msg) + strlen(out) + 1;
+		sendbuf = (char*) malloc(sendbufsize);
+		snprintf(sendbuf, sendbufsize, msg, out);
 		say(conn_info->clntinfo->clientsocket, sendbuf);
 		free(sendbuf);
 	}
@@ -156,18 +176,26 @@ int secsftp_cwd(char* args, struct conn_info_st* conn_info) {
 	switch (ret) {
 		case 0:
 			msg = "250 CWD command successful.\r\n";
-			sendbuf = msg;
 			conn_info->li->respcode = 250;
+			say(conn_info->clntinfo->clientsocket, msg);
 			break;
 		default:
-			msg = "550 %s: %s\r\n";
-			sendbuf = (char*) malloc(strlen(msg) 
-					+ strlen(sftp_out_buffer)
-					+ strlen(arrlist.arr[1]) + 1);
-			sprintf(sendbuf, msg, arrlist.arr[1], sftp_out_buffer);
-			conn_info->li->respcode = 550;
+			{
+				size_t sendbufsize;
+				msg = "550 %s: %s\r\n";
+				conn_info->li->respcode = 550;
+				sendbufsize = strlen(msg)
+						+ strlen(sftp_out_buffer)
+						+ strlen(arrlist.arr[1]) + 1;
+				sendbuf = (char*) malloc(sendbufsize);
+				snprintf(sendbuf, sendbufsize, msg,
+					arrlist.arr[1], sftp_out_buffer);
+				say(conn_info->clntinfo->clientsocket,
+					sendbuf);
+				free(sendbuf);
+				sendbuf = (char*) 0;
+			}
 	}
-	say(conn_info->clntinfo->clientsocket, sendbuf);
 	return CMD_HANDLED;
 }
 
@@ -197,6 +225,7 @@ int pipe_command(const char* replace, struct arrlist_t arrlist,
 		int fd[2], struct conn_info_st* conn_info) {
 	FILE* file =0;
 	int ret;
+	int arrlsize;
 	fd_set fset;
 	struct timeval tv = { 0, 0 };
 
@@ -211,9 +240,11 @@ int pipe_command(const char* replace, struct arrlist_t arrlist,
 
 	/* replace the FTP command with the Unix command, use a malloc()
 	 * like routine in order to be able to use free_arrlist() */
-	arrlist.arr[0] = (char*) realloc(arrlist.arr[0], strlen(replace) + 1);
+	arrlsize = strlen(replace) + 1;
+	arrlist.arr[0] = (char*) realloc(arrlist.arr[0], arrlsize);
 	enough_mem(arrlist.arr[0]);
-	strcpy(arrlist.arr[0], replace);
+	snprintf(arrlist.arr[0], arrlsize, "%s", replace);
+	strncpy(arrlist.arr[0], replace, strlen(replace));
 
 	file = fdopen(fd[1], "w");
 	ret = do_command(arrlist.arr, arrlist.amount, file);
@@ -272,6 +303,8 @@ int secsftp_mkd(char* args, struct conn_info_st* conn_info) {
 	if (ret == READY) {
 		/* An error occurred */
 		char* msg = "550 %s: %s\r\n";
+		int sendbufsize;
+
 		conn_info->li->respcode = 550;
 		if ((rb = read(fd[0], buffer, sizeof(buffer))) < 0) {
 			log(2, "Error reading from the pipe: %s", strerror(errno));
@@ -286,18 +319,20 @@ int secsftp_mkd(char* args, struct conn_info_st* conn_info) {
 		if (rb > 0 && buffer[rb-1] == '\n') {
 			buffer[rb-1] = '\0';
 		}
-		sendbuf = (char*) malloc(strlen(buffer)
+		sendbufsize = strlen(buffer)
 					+ strlen(msg)
 					+ strlen(arrlist.arr[1])
-					+ 1);
+					+ 1;
+		sendbuf = (char*) malloc(sendbufsize);
 		enough_mem(sendbuf);
-		sprintf(sendbuf, msg, arrlist.arr[1], buffer);
+		snprintf(sendbuf, sendbufsize, msg, arrlist.arr[1], buffer);
 	} else {
 		/* successful */
 		char* tmp = 0;
 		char* newdir = 0;
 		char* arr[2] = { "CWD", arrlist.arr[1] };
 		char* msg = "257 \"%s\" - successfully created.\r\n";
+		size_t sendbufsize;
 		conn_info->li->respcode = 257;
 
 		ret = do_pwd(NULL, 0);
@@ -317,11 +352,10 @@ int secsftp_mkd(char* args, struct conn_info_st* conn_info) {
 
 		free(arr[1]);
 		tmp = double_quote(newdir);
-		sendbuf = (char*) malloc(strlen(msg)
-					+ strlen(newdir)
-					+ 1);
+		sendbufsize = strlen(msg) + strlen(newdir) + 1;
+		sendbuf = (char*) malloc(sendbufsize);
 		enough_mem(sendbuf);
-		sprintf(sendbuf, msg, newdir);
+		snprintf(sendbuf, sendbufsize, msg, newdir);
 	}
 
 	say(conn_info->clntinfo->clientsocket, sendbuf);
@@ -341,6 +375,7 @@ int secsftp_rmd(char* args, struct conn_info_st* conn_info) {
 	int rb, ret;
 	char buffer[4096];
 	char* sendbuf =0;
+	size_t sendbufsize;
 
 	arrlist = split_args(args);
 	if (arrlist.amount != 2) {
@@ -378,12 +413,11 @@ int secsftp_rmd(char* args, struct conn_info_st* conn_info) {
 		if (rb > 0 && buffer[rb-1] == '\n') {
 			buffer[rb-1] = '\0';
 		}
-		sendbuf = (char*) malloc(strlen(buffer)
-					+ strlen(msg)
-					+ strlen(arrlist.arr[1])
-					+ 1);
+		sendbufsize = strlen(buffer) + strlen(msg)
+					+ strlen(arrlist.arr[1]) + 1;
+		sendbuf = (char*) malloc(sendbufsize);
 		enough_mem(sendbuf);
-		sprintf(sendbuf, msg, arrlist.arr[1], buffer);
+		snprintf(sendbuf, sendbufsize, msg, arrlist.arr[1], buffer);
 	} else {
 		/* successful */
 		conn_info->li->respcode = 250;
@@ -435,6 +469,7 @@ int secsftp_size_mdtm(char* args, struct conn_info_st* conn_info) {
 	if (ret == READY) {
 		/* We got a response */
 		char* msg = "213 %s\r\n";
+		size_t sendbufsize;
 		conn_info->li->respcode = 213;
 		if ((rb = read(fd[0], buffer, sizeof(buffer))) < 0) {
 			log(2, "Error reading from the pipe: %s", strerror(errno));
@@ -451,21 +486,20 @@ int secsftp_size_mdtm(char* args, struct conn_info_st* conn_info) {
 		}
 		if (*buffer != '-') {
 			/* Okay, we got a valid response */
-			sendbuf = (char*) malloc(strlen(msg)
-						+ strlen(buffer)
-						+ 1);
+			sendbufsize = strlen(msg) + strlen(buffer) + 1;
+			sendbuf = (char*) malloc(sendbufsize);
 			enough_mem(sendbuf);
-			sprintf(sendbuf, msg, buffer);
+			snprintf(sendbuf, sendbufsize, msg, buffer);
 		} else {
 			/* Possibly an error message */
 			msg = "550: %s: %s\r\n";
 			conn_info->li->respcode = 550;
-			sendbuf = (char*) malloc(strlen(msg)
-						+ strlen(buffer+1)
-						+ strlen(arrlist.arr[1])
-						+ 1);
+			sendbufsize = strlen(msg) + strlen(buffer+1)
+						+ strlen(arrlist.arr[1]) + 1;
+			sendbuf = (char*) malloc(sendbufsize);
 			enough_mem(sendbuf);
-			sprintf(sendbuf, msg, arrlist.arr[1], buffer+1);
+			snprintf(sendbuf, sendbufsize, msg,
+					arrlist.arr[1], buffer+1);
 		}
 	} else {
 		/* No response */
@@ -520,11 +554,12 @@ int secsftp_pass(char* args, struct conn_info_st* conn_info) {
 				return CMD_HANDLED;
 			} else {
 				char* succ_msg = "230 User %s logged in.\r\n";
-				char* msg = (char*) malloc(strlen(succ_msg) +
-					strlen(conn_info->lsforward->fwuser));
+				int sendbufsize = strlen(succ_msg) +
+					strlen(conn_info->lsforward->fwuser);
+				char* msg = (char*) malloc(sendbufsize);
 				conn_info->li->respcode = 230;
 				enough_mem(msg);
-				sprintf(msg, succ_msg,
+				snprintf(msg, sendbufsize, succ_msg,
 						conn_info->lsforward->fwuser);
 				say(cs, msg);
 				free(msg);
@@ -612,24 +647,27 @@ int secsftp_retr(char* args, struct conn_info_st* conn_info) {
 		close(parchld[0]);
 		close(s[0]);
 		conn_info->clntinfo->dataserversock = s[1];
-		size = ftp_readline(chldpar[0]);
+		size = readline(chldpar[0]);
 		log(9, "size: %s", size);
-		name = ftp_readline(chldpar[0]);
+		name = readline(chldpar[0]);
 		log(9, "name: %s", name);
 
 		if (strcmp(size, "-2") == 0) {
+			int sendbufsize;
 			error = 1;
 			/* This is the code for an error, the error can be
 			 * obtained from name */
 			msg = "550: %s: %s\r\n";
 			conn_info->li->respcode = 550;
-                        sendbuf = (char*) malloc(strlen(msg)
+			sendbufsize = strlen(msg)
 					+ strlen(arrlist.arr[1])
 					+ strlen(name)
-				       	+ 1);
-		       	enough_mem(sendbuf);
-		       	sprintf(sendbuf, msg, arrlist.arr[1], name);
-		       	say(conn_info->clntinfo->clientsocket, sendbuf);
+					+ 1;
+			sendbuf = (char*) malloc(sendbufsize);
+			enough_mem(sendbuf);
+			snprintf(sendbuf, sendbufsize, msg,
+					arrlist.arr[1], name);
+			say(conn_info->clntinfo->clientsocket, sendbuf);
 			free(sendbuf); sendbuf =0;
 		} else {
 			/* Everything went fine */
@@ -638,15 +676,16 @@ int secsftp_retr(char* args, struct conn_info_st* conn_info) {
 				say(conn_info->clntinfo->clientsocket, msglist);
 				sendbuf = 0;
 			} else {
-				sendbuf = (char*)malloc(strlen(msg) 
+				int sendbufsize = strlen(msg)
 							/* "ASCII" is shorter
 							 * than "binary" */
 							+ strlen("binary")
 							+ strlen(name)
 							+ strlen(size)
-							+ 1);
+							+ 1;
+				sendbuf = (char*)malloc(sendbufsize);
 				enough_mem(sendbuf);
-				sprintf(sendbuf, msg,
+				snprintf(sendbuf, sendbufsize, msg,
 						transfer_type == BINARY ?
 							"binary" : "ASCII",
 						name, size);
@@ -747,6 +786,7 @@ int secsftp_stor(char* args, struct conn_info_st* conn_info) {
 		exit(ret);
 	} else {
 		char buffer[4096];
+		size_t sendbufsize;
 		close(s[1]);
 		close(p[1]);
 		conn_info->clntinfo->dataserversock = s[0];
@@ -766,11 +806,13 @@ int secsftp_stor(char* args, struct conn_info_st* conn_info) {
 			/* An error has occurred */
 			msg = "550 %s: %s\r\n";
 			conn_info->li->respcode = 550;
-			sendbuf = (char*) malloc(strlen(msg)
-						+ strlen(arrlist.arr[1])
-						+ strlen(buffer)
-						+ 1);
-			sprintf(sendbuf, msg, arrlist.arr[1], buffer);
+			sendbufsize = strlen(msg)
+					+ strlen(arrlist.arr[1])
+					+ strlen(buffer)
+					+ 1;
+			sendbuf = (char*) malloc(sendbufsize);
+			snprintf(sendbuf, sendbufsize, msg,
+					arrlist.arr[1], buffer);
 			say(conn_info->clntinfo->clientsocket, sendbuf);
 			free(sendbuf);
 			free_arrlist(arrlist);
@@ -788,14 +830,16 @@ int secsftp_stor(char* args, struct conn_info_st* conn_info) {
 
 		msg = "150 Opening %s mode data connection for %s\r\n";
 		conn_info->li->respcode = 150;
-		sendbuf = (char*)malloc(strlen(msg) 
+		sendbufsize = strlen(msg)
 				/* "ASCII" is shorter than "binary" */
 				+ strlen("binary")
 				+ strlen(arrlist.arr[1])
-				+ 1);
+				+ 1;
+		sendbuf = (char*)malloc(sendbufsize);
 		enough_mem(sendbuf);
-		sprintf(sendbuf, msg, transfer_type == BINARY ?  "binary" 
-				: "ASCII", arrlist.arr[1]);
+		snprintf(sendbuf, sendbufsize,
+				msg, transfer_type == BINARY ?  "binary" 
+						: "ASCII", arrlist.arr[1]);
 		say(conn_info->clntinfo->clientsocket, sendbuf);
 
 		ret = transmitfile(conn_info->clntinfo);
