@@ -1,3 +1,21 @@
+/* 
+ * Copyright (C) 1999-2001 Joachim Wieland <joe@mcknight.de>
+ * 
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2 of the License, or (at your
+ * option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
+ */
+
 #include "jftpgw.h"
 #include "cmds.h"
 
@@ -45,13 +63,16 @@ int std_user(char* args, struct conn_info_st* conn_info) {
 
 	if (getuserdest(args, conn_info->clntinfo) == 0) {
 		char* sendbuf = 0;
+		size_t sendbufsize;
 		const char* passreq =
 			"331 Password required for %s.\r\n";
 		conn_info->usergiven = 1;
-		sendbuf = (char*) malloc(strlen(passreq) 
-				+ strlen(conn_info->lsforward->fwuser) + 1);
+		sendbufsize = strlen(passreq) 
+				+ strlen(conn_info->lsforward->fwuser)
+				+ 1;
+		sendbuf = (char*) malloc(sendbufsize);
 		enough_mem(sendbuf);
-		sprintf(sendbuf, passreq,
+		snprintf(sendbuf, sendbufsize, passreq,
 			conn_info->lsforward->fwuser);
 		say(cs, sendbuf);
 		free(sendbuf);
@@ -96,10 +117,15 @@ int std_pass(char* args, struct conn_info_st* conn_info) {
 				const char* errmsg = 
 					"500 Error connecting: %s\r\n";
 				int err = errno;
-				sendbuf = (char*) malloc(strlen(errmsg)
-					+ strlen(strerror(err)) + 1);
+				size_t sendbufsize;
+
+				sendbufsize = strlen(errmsg)
+					+ strlen(strerror(err))
+					+ 1;
+				sendbuf = (char*) malloc(sendbufsize);
 				enough_mem(sendbuf);
-				sprintf(sendbuf, errmsg, strerror(err));
+				snprintf(sendbuf, sendbufsize,
+						errmsg, strerror(err));
 				say(cs, sendbuf);
 				free(sendbuf);
 				sendbuf = 0;
@@ -107,7 +133,8 @@ int std_pass(char* args, struct conn_info_st* conn_info) {
 				return CMD_HANDLED;
 			}
 			if (ret < 0) {
-				/* login failed */
+				/* login failed - do not print an error
+				 * message */
 				conn_info->usergiven = 0;
 				conn_info->clntinfo->serversocket = ss = -1;
 			}
@@ -141,13 +168,19 @@ int std_pasv(char* args, struct conn_info_st* conn_info) {
 
 int std_port(char* args, struct conn_info_st* conn_info) {
 	int ret = 0;
+	size_t sendbufsize;
 	int cs = conn_info->clntinfo->clientsocket;
+	struct sockaddr_in sin;
 	char *answer = 0;
 	char *sendbuf = 0;
 
-	portcmd = (char*) malloc(strlen(args) + 1);
+	if (portcommandcheck(args, &sin, conn_info->clntinfo) < 0) {
+		/* an error was already reported */
+		return -1;
+	}
+	portcmd = strdup(args);
+	enough_mem(portcmd);
 
-	strcpy(portcmd, args);
 	if (conn_info->clntinfo->servermode == ACTIVE ||
 	    conn_info->clntinfo->servermode == ASCLIENT) {
 		ret = activeserver(&answer, conn_info->clntinfo);
@@ -156,10 +189,10 @@ int std_port(char* args, struct conn_info_st* conn_info) {
 			return CMD_HANDLED;
 		}
 		/* Repeat the answer of the server */
-		sendbuf = (char*) malloc(strlen(answer) + 3);
+		sendbufsize = strlen(answer) + 3;
+		sendbuf = (char*) malloc(sendbufsize);
 		enough_mem(sendbuf);
-		strcpy(sendbuf, answer);
-		strcat(sendbuf, "\r\n");
+		snprintf(sendbuf, sendbufsize, "%s\r\n", answer);
 		say(cs, sendbuf);
 		conn_info->li->respcode = respcode(sendbuf);
 		free(sendbuf);
@@ -177,10 +210,42 @@ int std_port(char* args, struct conn_info_st* conn_info) {
 
 
 int std_stor(char* args, struct conn_info_st* conn_info) {
+	/* chop of the "STOR " prefix */
+	char* space = strchr(args, ' ');
+	if (space) {
+		conn_info->li->name = space + 1;
+	} else {
+		conn_info->li->name = args;
+	}
 
+	conn_info->li->direction = 'i';
 	conn_info->clntinfo->mode = STOR;
 	return CMD_PASS;
-
 }
 
+int std_retr(char* args, struct conn_info_st* conn_info) {
+	/* chop of the "RETR " prefix */
+	char* space = strchr(args, ' ');
+	if (space) {
+		conn_info->li->name = space + 1;
+	} else {
+		conn_info->li->name = args;
+	}
+	conn_info->li->direction = 'o';
+	return CMD_PASS;
+}
+
+/* a simple function that determines the transfer mode (ascii or image) */
+
+int std_type(char* args, struct conn_info_st* conn_info) {
+	char* space = strrchr(args, ' ');
+	if (space) {
+		if (*(space + 1) == 'A' || *(space + 1) ==  'a') {
+			conn_info->li->type = 'a';
+		} else {
+			conn_info->li->type = 'b';
+		}
+	}
+	return CMD_PASS;
+}
 
